@@ -62,6 +62,11 @@ static inline int sha256_file(char *filename, sha256_digest *digest)
 #include <caml/bigarray.h>
 #include <caml/threads.h>
 
+#include <caml/version.h>
+#if OCAML_VERSION < 40600
+#define Bytes_val(x) ((unsigned char *) Bp_val(x))
+#endif
+
 #define GET_CTX_STRUCT(a) ((struct sha256_ctx *) a)
 
 CAMLexport value stub_sha256_init(value unit)
@@ -88,8 +93,8 @@ CAMLprim value stub_sha256_update_bigarray(value ctx, value buf)
 {
 	CAMLparam2(ctx, buf);
 	struct sha256_ctx ctx_dup;
-	unsigned char *data = Data_bigarray_val(buf);
-	size_t len = Bigarray_val(buf)->dim[0];
+	unsigned char *data = Caml_ba_data_val(buf);
+	size_t len = Caml_ba_array_val(buf)->dim[0];
 
 	ctx_dup = *GET_CTX_STRUCT(ctx);
 	caml_release_runtime_system();
@@ -152,7 +157,7 @@ CAMLprim value stub_sha256_to_bin(value digest)
 	CAMLlocal1(result);
 
 	result = caml_alloc_string(32);
-	sha256_to_bin((sha256_digest *) digest, String_val(result));
+	sha256_to_bin((sha256_digest *) digest, &Byte(result, 0));
 
 	CAMLreturn(result);
 }
@@ -163,7 +168,36 @@ CAMLprim value stub_sha256_to_hex(value digest)
 	CAMLlocal1(result);
 
 	result = caml_alloc_string(64);
-	sha256_to_hex((sha256_digest *) digest, String_val(result));
+	sha256_to_hex((sha256_digest *) digest, &Byte(result, 0));
+
+	CAMLreturn(result);
+}
+
+CAMLprim value stub_sha256_equal(value t1, value t2)
+{
+	CAMLparam2(t1, t2);
+	int b = memcmp((sha256_digest *) t1, (sha256_digest *) t2, sizeof(sha256_digest)) == 0;
+	CAMLreturn(Bool_val(b));
+}
+
+CAMLprim value stub_sha256_of_bin(value bin)
+{
+	CAMLparam1(bin);
+	CAMLlocal1(result);
+
+	result = caml_alloc(sizeof(sha256_digest), Abstract_tag);
+	sha256_of_bin((const char *) Bytes_val(bin), (sha256_digest *) result);
+
+	CAMLreturn(result);
+}
+
+CAMLprim value stub_sha256_of_hex(value hex)
+{
+	CAMLparam1(hex);
+	CAMLlocal1(result);
+
+	result = caml_alloc(sizeof(sha256_digest), Abstract_tag);
+	sha256_of_hex(String_val(hex), (sha256_digest *) result);
 
 	CAMLreturn(result);
 }
