@@ -62,6 +62,11 @@ static inline int sha512_file(char *filename, sha512_digest *digest)
 #include <caml/bigarray.h>
 #include <caml/threads.h>
 
+#include <caml/version.h>
+#if OCAML_VERSION < 40600
+#define Bytes_val(x) ((unsigned char *) Bp_val(x))
+#endif
+
 #define GET_CTX_STRUCT(a) ((struct sha512_ctx *) a)
 
 CAMLexport value stub_sha512_init(value unit)
@@ -88,8 +93,8 @@ CAMLprim value stub_sha512_update_bigarray(value ctx, value buf)
 {
 	CAMLparam2(ctx, buf);
 	struct sha512_ctx ctx_dup;
-	unsigned char *data = Data_bigarray_val(buf);
-	size_t len = Bigarray_val(buf)->dim[0];
+	unsigned char *data = Caml_ba_data_val(buf);
+	size_t len = Caml_ba_array_val(buf)->dim[0];
 
 	ctx_dup = *GET_CTX_STRUCT(ctx);
 	caml_release_runtime_system();
@@ -152,7 +157,7 @@ CAMLprim value stub_sha512_to_bin(value digest)
 	CAMLlocal1(result);
 
 	result = caml_alloc_string(64);
-	sha512_to_bin((sha512_digest *) digest, String_val(result));
+	sha512_to_bin((sha512_digest *) digest, &Byte(result, 0));
 
 	CAMLreturn(result);
 }
@@ -163,7 +168,36 @@ CAMLprim value stub_sha512_to_hex(value digest)
 	CAMLlocal1(result);
 
 	result = caml_alloc_string(128);
-	sha512_to_hex((sha512_digest *) digest, String_val(result));
+	sha512_to_hex((sha512_digest *) digest, &Byte(result, 0));
+
+	CAMLreturn(result);
+}
+
+CAMLprim value stub_sha512_equal(value t1, value t2)
+{
+	CAMLparam2(t1, t2);
+	int b = memcmp((sha512_digest *) t1, (sha512_digest *) t2, sizeof(sha512_digest)) == 0;
+	CAMLreturn(Bool_val(b));
+}
+
+CAMLprim value stub_sha512_of_bin(value bin)
+{
+	CAMLparam1(bin);
+	CAMLlocal1(result);
+
+	result = caml_alloc(sizeof(sha512_digest), Abstract_tag);
+	sha512_of_bin((const char *) Bytes_val(bin), (sha512_digest *) result);
+
+	CAMLreturn(result);
+}
+
+CAMLprim value stub_sha512_of_hex(value hex)
+{
+	CAMLparam1(hex);
+	CAMLlocal1(result);
+
+	result = caml_alloc(sizeof(sha512_digest), Abstract_tag);
+	sha512_of_hex(String_val(hex), (sha512_digest *) result);
 
 	CAMLreturn(result);
 }
